@@ -44,7 +44,7 @@ async def validate_block_body(
     fork_point_with_peak: Optional[uint32],
     get_block_generator: Callable[[BlockInfo], Awaitable[Optional[BlockGenerator]]],
     *,
-    validate_signature=True,
+    validate_signature: bool = True,
 ) -> Tuple[Optional[Err], Optional[NPCResult]]:
     """
     This assumes the header block has been completely validated.
@@ -147,12 +147,16 @@ async def validate_block_body(
         return Err.INVALID_REWARD_COINS, None
 
     removals: List[bytes32] = []
-    coinbase_additions: List[Coin] = list(expected_reward_coins)
-    additions: List[Coin] = []
+
+    # we store coins paired with their names in order to avoid computing the
+    # coin name multiple times, we store it next to the coin while validating
+    # the block
+    coinbase_additions: List[Tuple[Coin, bytes32]] = [(c, c.name()) for c in expected_reward_coins]
+    additions: List[Tuple[Coin, bytes32]] = []
     removals_puzzle_dic: Dict[bytes32, bytes32] = {}
     cost: uint64 = uint64(0)
 
-    # In header validation we check that timestamp is not more that 5 minutes into the future
+    # In header validation we check that timestamp is not more than 5 minutes into the future
     # 6. No transactions before INITIAL_TRANSACTION_FREEZE timestamp
     # (this test has been removed)
 
@@ -210,7 +214,8 @@ async def validate_block_body(
             removals.append(spend.coin_id)
             removals_puzzle_dic[spend.coin_id] = spend.puzzle_hash
             for puzzle_hash, amount, _ in spend.create_coin:
-                additions.append(Coin(spend.coin_id, puzzle_hash, uint64(amount)))
+                c = Coin(spend.coin_id, puzzle_hash, uint64(amount))
+                additions.append((c, c.name()))
     else:
         assert npc_result is None
 
@@ -222,8 +227,8 @@ async def validate_block_body(
     # 10. Check additions for max coin amount
     # Be careful to check for 64 bit overflows in other languages. This is the max 64 bit unsigned integer
     # We will not even reach here because Coins do type checking (uint64)
-    for coin in additions + coinbase_additions:
-        additions_dic[coin.name()] = coin
+    for coin, coin_name in additions + coinbase_additions:
+        additions_dic[coin_name] = coin
         if coin.amount < 0:
             return Err.COIN_AMOUNT_NEGATIVE, None
 
@@ -243,7 +248,7 @@ async def validate_block_body(
     # 12. The additions and removals must result in the correct filter
     byte_array_tx: List[bytearray] = []
 
-    for coin in additions + coinbase_additions:
+    for coin, _ in additions + coinbase_additions:
         byte_array_tx.append(bytearray(coin.puzzle_hash))
     for coin_name in removals:
         byte_array_tx.append(bytearray(coin_name))
@@ -256,7 +261,7 @@ async def validate_block_body(
         return Err.INVALID_TRANSACTIONS_FILTER_HASH, None
 
     # 13. Check for duplicate outputs in additions
-    addition_counter = collections.Counter(_.name() for _ in additions + coinbase_additions)
+    addition_counter = collections.Counter(coin_name for _, coin_name in additions + coinbase_additions)
     for k, v in addition_counter.items():
         if v > 1:
             return Err.DUPLICATE_OUTPUT, None
@@ -322,14 +327,16 @@ async def validate_block_body(
                 assert c_name not in removals_since_fork
                 removals_since_fork.add(c_name)
             for c in additions_in_curr:
-                assert c.name() not in additions_since_fork
+                coin_name = c.name()
+                assert coin_name not in additions_since_fork
                 assert curr.foliage_transaction_block is not None
-                additions_since_fork[c.name()] = (c, curr.height, curr.foliage_transaction_block.timestamp)
+                additions_since_fork[coin_name] = (c, curr.height, curr.foliage_transaction_block.timestamp)
 
             for coinbase_coin in curr.get_included_reward_coins():
-                assert coinbase_coin.name() not in additions_since_fork
+                coin_name = coinbase_coin.name()
+                assert coin_name not in additions_since_fork
                 assert curr.foliage_transaction_block is not None
-                additions_since_fork[coinbase_coin.name()] = (
+                additions_since_fork[coin_name] = (
                     coinbase_coin,
                     curr.height,
                     curr.foliage_transaction_block.timestamp,
@@ -340,6 +347,9 @@ async def validate_block_body(
             assert curr is not None
 
     removal_coin_records: Dict[bytes32, CoinRecord] = {}
+    # the removed coins we need to look up from the DB
+    # i.e. all non-ephemeral coins
+    removals_from_db: List[bytes32] = []
     for rem in removals:
         if rem in additions_dic:
             # Ephemeral coin
@@ -353,42 +363,60 @@ async def validate_block_body(
             )
             removal_coin_records[new_unspent.name] = new_unspent
         else:
-            unspent = await coin_store.get_coin_record(rem)
-            if unspent is not None and unspent.confirmed_block_index <= fork_h:
-                # Spending something in the current chain, confirmed before fork
-                # (We ignore all coins confirmed after fork)
-                if unspent.spent == 1 and unspent.spent_block_index <= fork_h:
-                    # Check for coins spent in an ancestor block
-                    return Err.DOUBLE_SPEND, None
-                removal_coin_records[unspent.name] = unspent
-            else:
-                # This coin is not in the current heaviest chain, so it must be in the fork
-                if rem not in additions_since_fork:
-                    # Check for spending a coin that does not exist in this fork
-                    log.error(f"Err.UNKNOWN_UNSPENT: COIN ID: {rem} NPC RESULT: {npc_result}")
-                    return Err.UNKNOWN_UNSPENT, None
-                new_coin, confirmed_height, confirmed_timestamp = additions_since_fork[rem]
-                new_coin_record: CoinRecord = CoinRecord(
-                    new_coin,
-                    confirmed_height,
-                    uint32(0),
-                    False,
-                    confirmed_timestamp,
-                )
-                removal_coin_records[new_coin_record.name] = new_coin_record
-
             # This check applies to both coins created before fork (pulled from coin_store),
             # and coins created after fork (additions_since_fork)
             if rem in removals_since_fork:
                 # This coin was spent in the fork
                 return Err.DOUBLE_SPEND_IN_FORK, None
+            removals_from_db.append(rem)
+
+    unspent_records = await coin_store.get_coin_records(removals_from_db)
+
+    # some coin spends we need to ensure exist in the fork branch. Both coins we
+    # can't find in the DB, but also coins that were spent after the fork point
+    look_in_fork: List[bytes32] = []
+    for unspent in unspent_records:
+        if unspent.confirmed_block_index <= fork_h:
+            # Spending something in the current chain, confirmed before fork
+            # (We ignore all coins confirmed after fork)
+            if unspent.spent == 1 and unspent.spent_block_index <= fork_h:
+                # Check for coins spent in an ancestor block
+                return Err.DOUBLE_SPEND, None
+            removal_coin_records[unspent.name] = unspent
+        else:
+            look_in_fork.append(unspent.name)
+
+    if len(unspent_records) != len(removals_from_db):
+        # some coins could not be found in the DB. We need to find out which
+        # ones and look for them in additions_since_fork
+        found: Set[bytes32] = set([u.name for u in unspent_records])
+        for rem in removals_from_db:
+            if rem in found:
+                continue
+            look_in_fork.append(rem)
+
+    for rem in look_in_fork:
+        # This coin is not in the current heaviest chain, so it must be in the fork
+        if rem not in additions_since_fork:
+            # Check for spending a coin that does not exist in this fork
+            log.error(f"Err.UNKNOWN_UNSPENT: COIN ID: {rem} NPC RESULT: {npc_result}")
+            return Err.UNKNOWN_UNSPENT, None
+        new_coin, confirmed_height, confirmed_timestamp = additions_since_fork[rem]
+        new_coin_record: CoinRecord = CoinRecord(
+            new_coin,
+            confirmed_height,
+            uint32(0),
+            False,
+            confirmed_timestamp,
+        )
+        removal_coin_records[new_coin_record.name] = new_coin_record
 
     removed = 0
     for unspent in removal_coin_records.values():
         removed += unspent.coin.amount
 
     added = 0
-    for coin in additions:
+    for coin, _ in additions:
         added += coin.amount
 
     # 16. Check that the total coin amount for added is <= removed
