@@ -85,5 +85,5 @@ class BlockRecord(Streamable):
             self.required_iters,
         )
 
-    def sp_total_iters(self, constants: ConsensusConstants):
-        return self.sp_sub_slot_total_iters(constants) + self.sp_iters(constants)
+    def sp_total_iters(self, constants: ConsensusConstants) -> uint128:
+        return uint128(self.sp_sub_slot_total_iters(constants) + self.sp_iters(constants))
