@@ -8,7 +8,6 @@ from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.types.coin_record import CoinRecord
 from chia.util.db_wrapper import DBWrapper2
 from chia.util.ints import uint32, uint64
-from chia.util.lru_cache import LRUCache
 from chia.util.chunks import chunks
 import time
 import logging
@@ -21,18 +20,14 @@ MAX_SQLITE_PARAMETERS = 900
 class CoinStore:
     """
     This object handles CoinRecords in DB.
-    A cache is maintained for quicker access to recent coins.
     """
 
-    coin_record_cache: LRUCache
-    cache_size: uint32
     db_wrapper: DBWrapper2
 
     @classmethod
-    async def create(cls, db_wrapper: DBWrapper2, cache_size: uint32 = uint32(60000)):
+    async def create(cls, db_wrapper: DBWrapper2):
         self = cls()
 
-        self.cache_size = cache_size
         self.db_wrapper = db_wrapper
 
         async with self.db_wrapper.write_db() as conn:
@@ -81,7 +76,6 @@ class CoinStore:
 
             await conn.execute("CREATE INDEX IF NOT EXISTS coin_parent_index on coin_record(coin_parent)")
 
-        self.coin_record_cache = LRUCache(cache_size)
         return self
 
     async def num_unspent(self) -> int:
@@ -161,10 +155,6 @@ class CoinStore:
 
     # Checks DB and DiffStores for CoinRecord with coin_name and returns it
     async def get_coin_record(self, coin_name: bytes32) -> Optional[CoinRecord]:
-        cached = self.coin_record_cache.get(coin_name)
-        if cached is not None:
-            return cached
-
         async with self.db_wrapper.read_db() as conn:
             async with conn.execute(
                 "SELECT confirmed_index, spent_index, coinbase, puzzle_hash, "
@@ -174,11 +164,40 @@ class CoinStore:
                 row = await cursor.fetchone()
                 if row is not None:
                     coin = self.row_to_coin(row)
-                    record = CoinRecord(coin, row[0], row[1], row[2], row[6])
-                    self.coin_record_cache.put(record.coin.name(), record)
-                    return record
+                    return CoinRecord(coin, row[0], row[1], row[2], row[6])
         return None
 
+    async def get_coin_records(self, names: List[bytes32]) -> List[CoinRecord]:
+        if len(names) == 0:
+            return []
+
+        coins: List[CoinRecord] = []
+
+        async with self.db_wrapper.read_db() as conn:
+            cursors: List[Cursor] = []
+            for names_chunk in chunks(names, MAX_SQLITE_PARAMETERS):
+                names_db: Tuple[Any, ...]
+                if self.db_wrapper.db_version == 2:
+                    names_db = tuple(names_chunk)
+                else:
+                    names_db = tuple([n.hex() for n in names_chunk])
+                cursors.append(
+                    await conn.execute(
+                        f"SELECT confirmed_index, spent_index, coinbase, puzzle_hash, "
+                        f"coin_parent, amount, timestamp FROM coin_record "
+                        f'WHERE coin_name in ({",".join(["?"] * len(names_db))}) ',
+                        names_db,
+                    )
+                )
+
+            for cursor in cursors:
+                for row in await cursor.fetchall():
+                    coin = self.row_to_coin(row)
+                    record = CoinRecord(coin, row[0], row[1], row[2], row[6])
+                    coins.append(record)
+
+        return coins
+
     async def get_coins_added_at_height(self, height: uint32) -> List[CoinRecord]:
         async with self.db_wrapper.read_db() as conn:
             async with conn.execute(
@@ -409,25 +428,9 @@ class CoinStore:
         Note that block_index can be negative, in which case everything is rolled back
         Returns the list of coin records that have been modified
         """
-        # Update memory cache
-        delete_queue: List[bytes32] = []
-        for coin_name, coin_record in list(self.coin_record_cache.cache.items()):
-            if int(coin_record.spent_block_index) > block_index:
-                new_record = CoinRecord(
-                    coin_record.coin,
-                    coin_record.confirmed_block_index,
-                    uint32(0),
-                    coin_record.coinbase,
-                    coin_record.timestamp,
-                )
-                self.coin_record_cache.put(coin_record.coin.name(), new_record)
-            if int(coin_record.confirmed_block_index) > block_index:
-                delete_queue.append(coin_name)
-
-        for coin_name in delete_queue:
-            self.coin_record_cache.remove(coin_name)
 
         coin_changes: Dict[bytes32, CoinRecord] = {}
+        # Add coins that are confirmed in the reverted blocks to the list of updated coins.
         async with self.db_wrapper.write_db() as conn:
             async with conn.execute(
                 "SELECT confirmed_index, spent_index, coinbase, puzzle_hash, "
@@ -439,12 +442,13 @@ class CoinStore:
                     record = CoinRecord(coin, uint32(0), row[1], row[2], uint64(0))
                     coin_changes[record.name] = record
 
-            # Delete from storage
+            # Delete reverted blocks from storage
             await conn.execute("DELETE FROM coin_record WHERE confirmed_index>?", (block_index,))
 
+            # Add coins that are confirmed in the reverted blocks to the list of changed coins.
             async with conn.execute(
                 "SELECT confirmed_index, spent_index, coinbase, puzzle_hash, "
-                "coin_parent, amount, timestamp FROM coin_record WHERE confirmed_index>?",
+                "coin_parent, amount, timestamp FROM coin_record WHERE spent_index>?",
                 (block_index,),
             ) as cursor:
                 for row in await cursor.fetchall():
@@ -461,13 +465,12 @@ class CoinStore:
                 )
         return list(coin_changes.values())
 
-    # Store CoinRecord in DB and ram cache
+    # Store CoinRecord in DB
     async def _add_coin_records(self, records: List[CoinRecord]) -> None:
 
         if self.db_wrapper.db_version == 2:
             values2 = []
             for record in records:
-                self.coin_record_cache.put(record.coin.name(), record)
                 values2.append(
                     (
                         record.coin.name(),
@@ -489,7 +492,6 @@ class CoinStore:
         else:
             values = []
             for record in records:
-                self.coin_record_cache.put(record.coin.name(), record)
                 values.append(
                     (
                         record.coin.name().hex(),
@@ -514,20 +516,14 @@ class CoinStore:
     async def _set_spent(self, coin_names: List[bytes32], index: uint32):
 
         assert len(coin_names) == 0 or index > 0
-        # if this coin is in the cache, mark it as spent in there
+
+        if len(coin_names) == 0:
+            return
+
         updates = []
         for coin_name in coin_names:
-            r = self.coin_record_cache.get(coin_name)
-            if r is not None:
-                if r.spent_block_index != uint32(0):
-                    raise ValueError(f"Coin already spent in cache: {coin_name}")
-
-                self.coin_record_cache.put(
-                    r.name, CoinRecord(r.coin, r.confirmed_block_index, index, r.coinbase, r.timestamp)
-                )
             updates.append((index, self.maybe_to_hex(coin_name)))
 
-        assert len(updates) == len(coin_names)
         async with self.db_wrapper.write_db() as conn:
             if self.db_wrapper.db_version == 2:
                 ret: Cursor = await conn.executemany(
