@@ -124,6 +124,15 @@ progress = {
 def install_bladebit(root_path):
     if is_bladebit_supported():
         print("Installing dependencies.")
+        run_command(
+            [
+                "sudo",
+                "apt",
+                "update",
+                "-y",
+            ],
+            "Could not update get package information from apt",
+        )
         run_command(
             [
                 "sudo",
@@ -134,6 +143,7 @@ def install_bladebit(root_path):
                 "cmake",
                 "libnuma-dev",
                 "git",
+                "libgmp-dev",
             ],
             "Could not install dependencies",
         )
