@@ -43,7 +43,7 @@ from chia.pools.pool_puzzles import (
     get_delayed_puz_info_from_launcher_spend,
 )
 
-from chia.util.ints import uint8, uint32, uint64
+from chia.util.ints import uint8, uint32, uint64, uint128
 from chia.wallet.derive_keys import (
     find_owner_sk,
 )
@@ -61,6 +61,7 @@ class PoolWallet:
     MINIMUM_INITIAL_BALANCE = 1
     MINIMUM_RELATIVE_LOCK_HEIGHT = 5
     MAXIMUM_RELATIVE_LOCK_HEIGHT = 1000
+    DEFAULT_MAX_CLAIM_SPENDS = 100
 
     wallet_state_manager: Any
     log: logging.Logger
@@ -127,7 +128,7 @@ class PoolWallet:
     @classmethod
     def _verify_self_pooled(cls, state) -> Optional[str]:
         err = ""
-        if state.pool_url != "":
+        if state.pool_url not in [None, ""]:
             err += " Unneeded pool_url for self-pooling"
 
         if state.relative_lock_height != 0:
@@ -326,6 +327,7 @@ class PoolWallet:
         block_spends: List[CoinSpend],
         block_height: uint32,
         in_transaction: bool,
+        *,
         name: str = None,
     ):
         """
@@ -778,13 +780,21 @@ class PoolWallet:
         travel_tx, fee_tx = await self.generate_travel_transactions(fee)
         return total_fee, travel_tx, fee_tx
 
-    async def claim_pool_rewards(self, fee: uint64) -> Tuple[TransactionRecord, Optional[TransactionRecord]]:
+    async def claim_pool_rewards(
+        self, fee: uint64, max_spends_in_tx: Optional[int]
+    ) -> Tuple[TransactionRecord, Optional[TransactionRecord]]:
         # Search for p2_puzzle_hash coins, and spend them with the singleton
         if await self.have_unconfirmed_transaction():
             raise ValueError(
                 "Cannot claim due to unconfirmed transaction. If this is stuck, delete the unconfirmed transaction."
             )
 
+        if max_spends_in_tx is None:
+            max_spends_in_tx = self.DEFAULT_MAX_CLAIM_SPENDS
+        elif max_spends_in_tx <= 0:
+            self.log.info(f"Bad max_spends_in_tx value of {max_spends_in_tx}. Set to {self.DEFAULT_MAX_CLAIM_SPENDS}.")
+            max_spends_in_tx = self.DEFAULT_MAX_CLAIM_SPENDS
+
         unspent_coin_records: List[CoinRecord] = list(
             await self.wallet_state_manager.coin_store.get_unspent_coins_for_wallet(self.wallet_id)
         )
@@ -807,13 +817,20 @@ class PoolWallet:
         all_spends: List[CoinSpend] = []
         total_amount = 0
 
-        current_coin_record = None
+        # The coins being claimed are gathered into the `SpendBundle`, :absorb_spend:
+        # We use an announcement in the fee spend to ensure that the claim spend is spent in the same block as the fee
+        # We only need to do this for one of the coins, because each `SpendBundle` can only be spent as a unit
+
+        first_coin_record = None
         for coin_record in unspent_coin_records:
             if coin_record.coin not in coin_to_height_farmed:
                 continue
-            current_coin_record = coin_record
-            if len(all_spends) >= 100:
-                # Limit the total number of spends, so it fits into the block
+            if first_coin_record is None:
+                first_coin_record = coin_record
+            if len(all_spends) >= max_spends_in_tx:
+                # Limit the total number of spends, so the SpendBundle fits into the block
+                self.log.info(f"pool wallet truncating absorb to {max_spends_in_tx} spends to fit into block")
+                print(f"pool wallet truncating absorb to {max_spends_in_tx} spends to fit into block")
                 break
             absorb_spend: List[CoinSpend] = create_absorb_spend(
                 last_solution,
@@ -830,7 +847,7 @@ class PoolWallet:
             self.log.info(
                 f"Farmer coin: {coin_record.coin} {coin_record.coin.name()} {coin_to_height_farmed[coin_record.coin]}"
             )
-        if len(all_spends) == 0 or current_coin_record is None:
+        if len(all_spends) == 0 or first_coin_record is None:
             raise ValueError("Nothing to claim, no unspent coinbase rewards")
 
         claim_spend: SpendBundle = SpendBundle(all_spends, G2Element())
@@ -840,7 +857,7 @@ class PoolWallet:
 
         fee_tx = None
         if fee > 0:
-            absorb_announce = Announcement(current_coin_record.coin.name(), b"$")
+            absorb_announce = Announcement(first_coin_record.coin.name(), b"$")
             fee_tx = await self.generate_fee_transaction(fee, coin_announcements=[absorb_announce])
             full_spend = SpendBundle.aggregate([fee_tx.spend_bundle, claim_spend])
 
@@ -919,25 +936,25 @@ class PoolWallet:
         )
         return len(unconfirmed) > 0
 
-    async def get_confirmed_balance(self, _=None) -> uint64:
-        amount: uint64 = uint64(0)
+    async def get_confirmed_balance(self, _=None) -> uint128:
+        amount: uint128 = uint128(0)
         if (await self.get_current_state()).current.state == SELF_POOLING:
             unspent_coin_records: List[WalletCoinRecord] = list(
                 await self.wallet_state_manager.coin_store.get_unspent_coins_for_wallet(self.wallet_id)
             )
             for record in unspent_coin_records:
                 if record.coinbase:
-                    amount = uint64(amount + record.coin.amount)
+                    amount = uint128(amount + record.coin.amount)
         return amount
 
-    async def get_unconfirmed_balance(self, record_list=None) -> uint64:
+    async def get_unconfirmed_balance(self, record_list=None) -> uint128:
         return await self.get_confirmed_balance(record_list)
 
-    async def get_spendable_balance(self, record_list=None) -> uint64:
+    async def get_spendable_balance(self, record_list=None) -> uint128:
         return await self.get_confirmed_balance(record_list)
 
     async def get_pending_change_balance(self) -> uint64:
         return uint64(0)
 
-    async def get_max_send_amount(self, record_list=None) -> uint64:
-        return uint64(0)
+    async def get_max_send_amount(self, record_list=None) -> uint128:
+        return uint128(0)
