@@ -97,6 +97,14 @@ rate_limits_other = {
     ProtocolMessageTypes.farm_new_block: RLSettings(200, 200),
     ProtocolMessageTypes.request_plots: RLSettings(10, 10 * 1024 * 1024),
     ProtocolMessageTypes.respond_plots: RLSettings(10, 100 * 1024 * 1024),
+    ProtocolMessageTypes.plot_sync_start: RLSettings(1000, 100 * 1024 * 1024),
+    ProtocolMessageTypes.plot_sync_loaded: RLSettings(1000, 100 * 1024 * 1024),
+    ProtocolMessageTypes.plot_sync_removed: RLSettings(1000, 100 * 1024 * 1024),
+    ProtocolMessageTypes.plot_sync_invalid: RLSettings(1000, 100 * 1024 * 1024),
+    ProtocolMessageTypes.plot_sync_keys_missing: RLSettings(1000, 100 * 1024 * 1024),
+    ProtocolMessageTypes.plot_sync_duplicates: RLSettings(1000, 100 * 1024 * 1024),
+    ProtocolMessageTypes.plot_sync_done: RLSettings(1000, 100 * 1024 * 1024),
+    ProtocolMessageTypes.plot_sync_response: RLSettings(3000, 100 * 1024 * 1024),
     ProtocolMessageTypes.coin_state_update: RLSettings(1000, 100 * 1024 * 1024),
     ProtocolMessageTypes.register_interest_in_puzzle_hash: RLSettings(1000, 100 * 1024 * 1024),
     ProtocolMessageTypes.respond_to_ph_update: RLSettings(1000, 100 * 1024 * 1024),
