@@ -1,6 +1,7 @@
 from typing import List, Optional
 
 from chia.consensus.block_record import BlockRecord
+from chia.consensus.multiprocess_validation import PreValidationResult
 from chia.full_node.full_node_api import FullNodeAPI
 from chia.protocols.full_node_protocol import RespondBlock
 from chia.simulator.simulator_protocol import FarmNewBlockProtocol, ReorgProtocol
@@ -44,12 +45,18 @@ class FullNodeSimulator(FullNodeAPI):
 
     @api_request
     async def farm_new_transaction_block(self, request: FarmNewBlockProtocol):
-        async with self.full_node.blockchain.lock:
+        async with self.full_node._blockchain_lock_high_priority:
             self.log.info("Farming new block!")
             current_blocks = await self.get_all_full_blocks()
             if len(current_blocks) == 0:
                 genesis = self.bt.get_consecutive_blocks(uint8(1))[0]
-                await self.full_node.blockchain.receive_block(genesis)
+                pre_validation_results: List[
+                    PreValidationResult
+                ] = await self.full_node.blockchain.pre_validate_blocks_multiprocessing(
+                    [genesis], {}, validate_signatures=True
+                )
+                assert pre_validation_results is not None
+                await self.full_node.blockchain.receive_block(genesis, pre_validation_results[0])
 
             peak = self.full_node.blockchain.get_peak()
             assert peak is not None
@@ -80,12 +87,18 @@ class FullNodeSimulator(FullNodeAPI):
 
     @api_request
     async def farm_new_block(self, request: FarmNewBlockProtocol):
-        async with self.full_node.blockchain.lock:
+        async with self.full_node._blockchain_lock_high_priority:
             self.log.info("Farming new block!")
             current_blocks = await self.get_all_full_blocks()
             if len(current_blocks) == 0:
                 genesis = self.bt.get_consecutive_blocks(uint8(1))[0]
-                await self.full_node.blockchain.receive_block(genesis)
+                pre_validation_results: List[
+                    PreValidationResult
+                ] = await self.full_node.blockchain.pre_validate_blocks_multiprocessing(
+                    [genesis], {}, validate_signatures=True
+                )
+                assert pre_validation_results is not None
+                await self.full_node.blockchain.receive_block(genesis, pre_validation_results[0])
 
             peak = self.full_node.blockchain.get_peak()
             assert peak is not None
