@@ -90,6 +90,25 @@ async def spawn_all_processes(config: Dict, net_config: Dict):
     await asyncio.gather(*awaitables)
 
 
+def signal_received():
+    asyncio.create_task(kill_processes())
+
+
+async def async_main(config, net_config):
+    loop = asyncio.get_running_loop()
+
+    try:
+        loop.add_signal_handler(signal.SIGINT, signal_received)
+        loop.add_signal_handler(signal.SIGTERM, signal_received)
+    except NotImplementedError:
+        log.info("signal handlers unsupported")
+
+    try:
+        await spawn_all_processes(config, net_config)
+    finally:
+        log.info("Launcher fully closed.")
+
+
 def main():
     if os.name == "nt":
         log.info("Timelord launcher not supported on Windows.")
@@ -100,22 +119,7 @@ def main():
     config = net_config["timelord_launcher"]
     initialize_logging("TLauncher", config["logging"], root_path)
 
-    def signal_received():
-        asyncio.create_task(kill_processes())
-
-    loop = asyncio.get_event_loop()
-
-    try:
-        loop.add_signal_handler(signal.SIGINT, signal_received)
-        loop.add_signal_handler(signal.SIGTERM, signal_received)
-    except NotImplementedError:
-        log.info("signal handlers unsupported")
-
-    try:
-        loop.run_until_complete(spawn_all_processes(config, net_config))
-    finally:
-        log.info("Launcher fully closed.")
-        loop.close()
+    asyncio.run(async_main(config=config, net_config=net_config))
 
 
 if __name__ == "__main__":
