@@ -13,7 +13,7 @@ from functools import wraps
 from hashlib import pbkdf2_hmac
 from pathlib import Path
 from secrets import token_bytes
-from typing import Optional
+from typing import Any, Dict, Optional
 from watchdog.events import FileSystemEventHandler
 from watchdog.observers import Observer
 
@@ -36,9 +36,7 @@ def loads_keyring(method):
 
     @wraps(method)
     def inner(self, *args, **kwargs):
-        # Watchdog's event dispatch timing is unreliable on macOS. Force a file modification time check for macOS.
-        if sys.platform == "darwin":
-            self.check_if_keyring_file_modified()
+        self.check_if_keyring_file_modified()
 
         # Check the outer payload for 'data', and check if we have a decrypted cache (payload_cache)
         with self.load_keyring_lock:
@@ -110,6 +108,9 @@ class FileKeyring(FileSystemEventHandler):
         # that holds a dictionary of keys.
         data: <base64-encoded string of encrypted inner-payload>
 
+        # An optional passphrase hint
+        passphrase_hint: <cleartext string>
+
     The file is encrypted using ChaCha20Poly1305. The symmetric key is derived from the
     master passphrase using PBKDF2. The nonce is updated each time the file is written-to.
     The salt is updated each time the master passphrase is changed.
@@ -152,6 +153,9 @@ class FileKeyring(FileSystemEventHandler):
         self.load_keyring_lock = threading.RLock()
         self.keyring_last_mod_time = None
 
+        # Key/value pairs to set on the outer payload on the next write
+        self.outer_payload_properties_for_next_write: Dict[str, Any] = {}
+
         if not self.keyring_path.exists():
             # Super simple payload if starting from scratch
             outer_payload = FileKeyring.default_outer_payload()
@@ -163,12 +167,10 @@ class FileKeyring(FileSystemEventHandler):
         self.setup_keyring_file_watcher()
 
     def setup_keyring_file_watcher(self):
-        observer = Observer()
-        # recursive=True necessary for macOS support
-        observer.schedule(self, self.keyring_path.parent, recursive=True)
-        observer.start()
-
         self.keyring_observer = Observer()
+        # recursive=True necessary for macOS support
+        self.keyring_observer.schedule(self, self.keyring_path.parent, recursive=True)
+        self.keyring_observer.start()
 
     def cleanup_keyring_file_watcher(self):
         if getattr(self, "keyring_observer"):
@@ -422,6 +424,10 @@ class FileKeyring(FileSystemEventHandler):
             "data": base64.b64encode(encrypted_inner_payload).decode("utf-8"),
         }
 
+        # Merge in other properties like "passphrase_hint"
+        outer_payload.update(self.outer_payload_properties_for_next_write)
+        self.outer_payload_properties_for_next_write = {}
+
         self.write_data_to_keyring(outer_payload)
 
         # Update our cached payload
@@ -444,3 +450,25 @@ class FileKeyring(FileSystemEventHandler):
 
         if not self.salt:
             self.salt = FileKeyring.generate_salt()
+
+    def get_passphrase_hint(self) -> Optional[str]:
+        """
+        Return the passphrase hint (if set). The hint data may not yet be written to the keyring, so we
+        return the hint data either from the staging dict (outer_payload_properties_for_next_write), or
+        from outer_payload_cache (loaded from the keyring)
+        """
+        passphrase_hint: Optional[str] = self.outer_payload_properties_for_next_write.get("passphrase_hint", None)
+        if passphrase_hint is None:
+            passphrase_hint = self.outer_payload_cache.get("passphrase_hint", None)
+        return passphrase_hint
+
+    def set_passphrase_hint(self, passphrase_hint: Optional[str]) -> None:
+        """
+        Store the new passphrase hint in the staging dict (outer_payload_properties_for_next_write) to
+        be written-out on the next write to the keyring.
+        """
+        assert self.outer_payload_properties_for_next_write is not None
+        if passphrase_hint is not None and len(passphrase_hint) > 0:
+            self.outer_payload_properties_for_next_write["passphrase_hint"] = passphrase_hint
+        elif "passphrase_hint" in self.outer_payload_properties_for_next_write:
+            del self.outer_payload_properties_for_next_write["passphrase_hint"]
