@@ -2,20 +2,22 @@ import dataclasses
 import logging
 import time
 import traceback
-from typing import Any, Dict, List, Optional, Tuple, Union, Set
+from typing import Any, Dict, List, Optional, Set, Tuple, Union
 
 from chia.protocols.wallet_protocol import CoinState
-from chia.types.blockchain_format.coin import Coin
+from chia.types.blockchain_format.coin import Coin, coin_as_list
 from chia.types.blockchain_format.program import Program
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.types.spend_bundle import SpendBundle
 from chia.util.db_wrapper import DBWrapper
 from chia.util.hash import std_hash
 from chia.util.ints import uint32, uint64
-from chia.wallet.cat_wallet.cat_wallet import CATWallet
+from chia.wallet.nft_wallet.nft_wallet import NFTWallet
+from chia.wallet.outer_puzzles import AssetType
 from chia.wallet.payment import Payment
+from chia.wallet.puzzle_drivers import PuzzleInfo
 from chia.wallet.trade_record import TradeRecord
-from chia.wallet.trading.offer import Offer, NotarizedPayment
+from chia.wallet.trading.offer import NotarizedPayment, Offer
 from chia.wallet.trading.trade_status import TradeStatus
 from chia.wallet.trading.trade_store import TradeStore
 from chia.wallet.transaction_record import TransactionRecord
@@ -23,9 +25,48 @@ from chia.wallet.util.transaction_type import TransactionType
 from chia.wallet.util.wallet_types import WalletType
 from chia.wallet.wallet import Wallet
 from chia.wallet.wallet_coin_record import WalletCoinRecord
+from chia.wallet.puzzles.load_clvm import load_clvm
+
+OFFER_MOD = load_clvm("settlement_payments.clvm")
 
 
 class TradeManager:
+    """
+    This class is a driver for creating and accepting settlement_payments.clvm style offers.
+
+    By default, standard XCH is supported but to support other types of assets you must implement certain functions on
+    the asset's wallet as well as create a driver for its puzzle(s).  Here is a guide to integrating a new types of
+    assets with this trade manager:
+
+    Puzzle Drivers:
+      - See chia/wallet/outer_puzzles.py for a full description of how to build these
+      - The `solve` method must be able to be solved by a Solver that looks like this:
+            Solver(
+                {
+                    "coin": bytes
+                    "parent_spend": bytes
+                    "siblings": List[bytes]  # other coins of the same type being offered
+                    "sibling_spends": List[bytes]  # The parent spends for the siblings
+                    "sibling_puzzles": List[Program]  # The inner puzzles of the siblings (always OFFER_MOD)
+                    "sibling_solutions": List[Program]  # The inner solution of the siblings
+                }
+            )
+
+    Wallet:
+      - Segments in this code that call general wallet methods are highlighted by comments: # ATTENTION: new wallets
+      - To be able to be traded, a wallet must implement these methods on itself:
+        - generate_signed_transaction(...) -> List[TransactionRecord]  (See cat_wallet.py for full API)
+        - convert_puzzle_hash(puzzle_hash: bytes32) -> bytes32  # Converts a puzzlehash from outer to inner puzzle
+        - get_puzzle_info(asset_id: bytes32) -> PuzzleInfo
+        - get_coins_to_offer(asset_id: bytes32, amount: uint64) -> Set[Coin]
+      - If you would like assets from your wallet to be referenced with just a wallet ID, you must also implement:
+        - get_asset_id() -> bytes32
+      - Finally, you must make sure that your wallet will respond appropriately when these WSM methods are called:
+        - get_wallet_for_puzzle_info(puzzle_info: PuzzleInfo) -> <Your wallet>
+        - create_wallet_for_puzzle_info(..., puzzle_info: PuzzleInfo) -> <Your wallet>  (See cat_wallet.py for full API)
+        - get_wallet_for_asset_id(asset_id: bytes32) -> <Your wallet>
+    """
+
     wallet_state_manager: Any
     log: logging.Logger
     trade_store: TradeStore
@@ -186,14 +227,13 @@ class TradeManager:
 
             if wallet is None:
                 continue
-            new_ph = await wallet.get_new_puzzlehash()
-            # This should probably not switch on whether or not we're spending a CAT but it has to for now
-            if wallet.type() == WalletType.CAT:
-                txs = await wallet.generate_signed_transaction(
-                    [coin.amount], [new_ph], fee=fee_to_pay, coins={coin}, ignore_max_send_amount=True
-                )
-                all_txs.extend(txs)
+
+            if wallet.type() == WalletType.NFT:
+                new_ph = await wallet.wallet_state_manager.main_wallet.get_new_puzzlehash()
             else:
+                new_ph = await wallet.get_new_puzzlehash()
+            # This should probably not switch on whether or not we're spending a XCH but it has to for now
+            if wallet.type() == WalletType.STANDARD_WALLET:
                 if fee_to_pay > coin.amount:
                     selected_coins: Set[Coin] = await wallet.select_coins(
                         uint64(fee_to_pay - coin.amount),
@@ -210,6 +250,12 @@ class TradeManager:
                     ignore_max_send_amount=True,
                 )
                 all_txs.append(tx)
+            else:
+                # ATTENTION: new_wallets
+                txs = await wallet.generate_signed_transaction(
+                    [coin.amount], [new_ph], fee=fee_to_pay, coins={coin}, ignore_max_send_amount=True
+                )
+                all_txs.extend(txs)
             fee_to_pay = uint64(0)
 
             cancellation_addition = Coin(coin.name(), new_ph, coin.amount)
@@ -246,9 +292,15 @@ class TradeManager:
         self.wallet_state_manager.state_changed("offer_added")
 
     async def create_offer_for_ids(
-        self, offer: Dict[Union[int, bytes32], int], fee: uint64 = uint64(0), validate_only: bool = False
+        self,
+        offer: Dict[Union[int, bytes32], int],
+        driver_dict: Optional[Dict[bytes32, PuzzleInfo]] = None,
+        fee: uint64 = uint64(0),
+        validate_only: bool = False,
     ) -> Tuple[bool, Optional[TradeRecord], Optional[str]]:
-        success, created_offer, error = await self._create_offer_for_ids(offer, fee=fee)
+        if driver_dict is None:
+            driver_dict = {}
+        success, created_offer, error = await self._create_offer_for_ids(offer, driver_dict, fee=fee)
         if not success or created_offer is None:
             raise Exception(f"Error creating offer: {error}")
 
@@ -273,14 +325,20 @@ class TradeManager:
         return success, trade_offer, error
 
     async def _create_offer_for_ids(
-        self, offer_dict: Dict[Union[int, bytes32], int], fee: uint64 = uint64(0)
+        self,
+        offer_dict: Dict[Union[int, bytes32], int],
+        driver_dict: Optional[Dict[bytes32, PuzzleInfo]] = None,
+        fee: uint64 = uint64(0),
     ) -> Tuple[bool, Optional[Offer], Optional[str]]:
         """
         Offer is dictionary of wallet ids and amount
         """
+        if driver_dict is None:
+            driver_dict = {}
         try:
-            coins_to_offer: Dict[uint32, List[Coin]] = {}
+            coins_to_offer: Dict[Union[int, bytes32], List[Coin]] = {}
             requested_payments: Dict[Optional[bytes32], List[Payment]] = {}
+            offer_dict_no_ints: Dict[Optional[bytes32], int] = {}
             for id, amount in offer_dict.items():
                 if amount > 0:
                     if isinstance(id, int):
@@ -288,44 +346,99 @@ class TradeManager:
                         wallet = self.wallet_state_manager.wallets[wallet_id]
                         p2_ph: bytes32 = await wallet.get_new_puzzlehash()
                         if wallet.type() == WalletType.STANDARD_WALLET:
-                            key: Optional[bytes32] = None
+                            asset_id: Optional[bytes32] = None
                             memos: List[bytes] = []
-                        elif wallet.type() == WalletType.CAT:
-                            key = bytes32(bytes.fromhex(wallet.get_asset_id()))
+                        elif callable(getattr(wallet, "get_asset_id", None)):  # ATTENTION: new wallets
+                            asset_id = bytes32(bytes.fromhex(wallet.get_asset_id()))
                             memos = [p2_ph]
                         else:
-                            raise ValueError(f"Offers are not implemented for {wallet.type()}")
+                            raise ValueError(
+                                f"Cannot request assets from wallet id {wallet.id()} without more information"
+                            )
                     else:
                         p2_ph = await self.wallet_state_manager.main_wallet.get_new_puzzlehash()
-                        key = id
+                        asset_id = id
+                        wallet = await self.wallet_state_manager.get_wallet_for_asset_id(asset_id.hex())
                         memos = [p2_ph]
-                    requested_payments[key] = [Payment(p2_ph, uint64(amount), memos)]
+                    requested_payments[asset_id] = [Payment(p2_ph, uint64(amount), memos)]
                 elif amount < 0:
-                    assert isinstance(id, int)
-                    wallet_id = uint32(id)
-                    wallet = self.wallet_state_manager.wallets[wallet_id]
-                    balance = await wallet.get_confirmed_balance()
-                    if balance < abs(amount):
-                        raise Exception(f"insufficient funds in wallet {wallet_id}")
-                    coins_to_offer[wallet_id] = await wallet.select_coins(uint64(abs(amount)))
+                    if isinstance(id, int):
+                        wallet_id = uint32(id)
+                        wallet = self.wallet_state_manager.wallets[wallet_id]
+                        if wallet.type() == WalletType.STANDARD_WALLET:
+                            asset_id = None
+                        elif callable(getattr(wallet, "get_asset_id", None)):  # ATTENTION: new wallets
+                            asset_id = bytes32(bytes.fromhex(wallet.get_asset_id()))
+                        else:
+                            raise ValueError(
+                                f"Cannot offer assets from wallet id {wallet.id()} without more information"
+                            )
+                    else:
+                        asset_id = id
+                        wallet = await self.wallet_state_manager.get_wallet_for_asset_id(asset_id.hex())
+                    if not callable(getattr(wallet, "get_coins_to_offer", None)):  # ATTENTION: new wallets
+                        raise ValueError(f"Cannot offer coins from wallet id {wallet.id()}")
+                    coins_to_offer[id] = await wallet.get_coins_to_offer(asset_id, uint64(abs(amount)))
                 elif amount == 0:
                     raise ValueError("You cannot offer nor request 0 amount of something")
 
+                offer_dict_no_ints[asset_id] = amount
+
+                if asset_id is not None and wallet is not None:
+                    if callable(getattr(wallet, "get_puzzle_info", None)):
+                        puzzle_driver: PuzzleInfo = wallet.get_puzzle_info(asset_id)
+                        if asset_id in driver_dict and driver_dict[asset_id] != puzzle_driver:
+                            # ignore the case if we're an nft transfering the did owner
+                            if self.check_for_owner_change_in_drivers(puzzle_driver, driver_dict[asset_id]):
+                                driver_dict[asset_id] = puzzle_driver
+                            else:
+                                raise ValueError(
+                                    f"driver_dict specified {driver_dict[asset_id]}, was expecting {puzzle_driver}"
+                                )
+                        else:
+                            driver_dict[asset_id] = puzzle_driver
+                    else:
+                        raise ValueError(f"Wallet for asset id {asset_id} is not properly integrated with TradeManager")
+
+            potential_special_offer: Optional[Offer] = await self.check_for_special_offer_making(
+                offer_dict_no_ints,
+                driver_dict,
+                fee,
+            )
+
+            if potential_special_offer is not None:
+                return True, potential_special_offer, None
+
             all_coins: List[Coin] = [c for coins in coins_to_offer.values() for c in coins]
             notarized_payments: Dict[Optional[bytes32], List[NotarizedPayment]] = Offer.notarize_payments(
                 requested_payments, all_coins
             )
-            announcements_to_assert = Offer.calculate_announcements(notarized_payments)
+            announcements_to_assert = Offer.calculate_announcements(notarized_payments, driver_dict)
 
             all_transactions: List[TransactionRecord] = []
             fee_left_to_pay: uint64 = fee
-            for wallet_id, selected_coins in coins_to_offer.items():
-                wallet = self.wallet_state_manager.wallets[wallet_id]
-                # This should probably not switch on whether or not we're spending a CAT but it has to for now
-
-                if wallet.type() == WalletType.CAT:
+            for id, selected_coins in coins_to_offer.items():
+                if isinstance(id, int):
+                    wallet = self.wallet_state_manager.wallets[id]
+                else:
+                    wallet = await self.wallet_state_manager.get_wallet_for_asset_id(id.hex())
+                # This should probably not switch on whether or not we're spending XCH but it has to for now
+                if wallet.type() == WalletType.STANDARD_WALLET:
+                    tx = await wallet.generate_signed_transaction(
+                        abs(offer_dict[id]),
+                        Offer.ph(),
+                        fee=fee_left_to_pay,
+                        coins=set(selected_coins),
+                        puzzle_announcements_to_consume=announcements_to_assert,
+                    )
+                    all_transactions.append(tx)
+                elif wallet.type() == WalletType.NFT:
+                    # This is to generate the tx for specific nft assets, i.e. not using
+                    # wallet_id as the selector which would select any coins from nft_wallet
+                    amounts = [coin.amount for coin in selected_coins]
                     txs = await wallet.generate_signed_transaction(
-                        [abs(offer_dict[int(wallet_id)])],
+                        # [abs(offer_dict[id])],
+                        amounts,
                         [Offer.ph()],
                         fee=fee_left_to_pay,
                         coins=set(selected_coins),
@@ -333,20 +446,23 @@ class TradeManager:
                     )
                     all_transactions.extend(txs)
                 else:
-                    tx = await wallet.generate_signed_transaction(
-                        abs(offer_dict[int(wallet_id)]),
-                        Offer.ph(),
+                    # ATTENTION: new_wallets
+                    txs = await wallet.generate_signed_transaction(
+                        [abs(offer_dict[id])],
+                        [Offer.ph()],
                         fee=fee_left_to_pay,
                         coins=set(selected_coins),
                         puzzle_announcements_to_consume=announcements_to_assert,
                     )
-                    all_transactions.append(tx)
+                    all_transactions.extend(txs)
 
                 fee_left_to_pay = uint64(0)
 
-            transaction_bundles: List[Optional[SpendBundle]] = [tx.spend_bundle for tx in all_transactions]
-            total_spend_bundle = SpendBundle.aggregate(list(filter(lambda b: b is not None, transaction_bundles)))
-            offer = Offer(notarized_payments, total_spend_bundle)
+            total_spend_bundle = SpendBundle.aggregate(
+                [x.spend_bundle for x in all_transactions if x.spend_bundle is not None]
+            )
+
+            offer = Offer(notarized_payments, total_spend_bundle, driver_dict)
             return True, offer, None
 
         except Exception as e:
@@ -358,13 +474,12 @@ class TradeManager:
 
         for key in offer.arbitrage():
             wsm = self.wallet_state_manager
-            wallet: Wallet = wsm.main_wallet
             if key is None:
                 continue
-            exists: Optional[Wallet] = await wsm.get_wallet_for_asset_id(key.hex())
+            # ATTENTION: new_wallets
+            exists: Optional[Wallet] = await wsm.get_wallet_for_puzzle_info(offer.driver_dict[key])
             if exists is None:
-                self.log.info(f"Creating wallet for asset ID: {key}")
-                await CATWallet.create_wallet_for_cat(wsm, wallet, key.hex())
+                await wsm.create_wallet_for_puzzle_info(offer.driver_dict[key])
 
     async def check_offer_validity(self, offer: Offer) -> bool:
         all_removals: List[Coin] = offer.bundle.removals()
@@ -398,7 +513,7 @@ class TradeManager:
                 wallet_id, _ = wallet_info
                 if addition.parent_coin_info in settlement_coin_ids:
                     wallet = self.wallet_state_manager.wallets[wallet_id]
-                    to_puzzle_hash = await wallet.convert_puzzle_hash(addition.puzzle_hash)
+                    to_puzzle_hash = await wallet.convert_puzzle_hash(addition.puzzle_hash)  # ATTENTION: new wallets
                     txs.append(
                         TransactionRecord(
                             confirmed_at_height=uint32(0),
@@ -432,13 +547,16 @@ class TradeManager:
                 removal_dict.setdefault(wallet_id, [])
                 removal_dict[wallet_id].append(removal)
 
+        all_removals: List[bytes32] = [r.name() for removals in removal_dict.values() for r in removals]
+
         for wid, grouped_removals in removal_dict.items():
             wallet = self.wallet_state_manager.wallets[wid]
             to_puzzle_hash = bytes32([1] * 32)  # We use all zeros to be clear not to send here
-            removal_tree_hash = Program.to([rem.as_list() for rem in grouped_removals]).get_tree_hash()
+            removal_tree_hash = Program.to([coin_as_list(rem) for rem in grouped_removals]).get_tree_hash()
             # We also need to calculate the sent amount
             removed: int = sum(c.amount for c in grouped_removals)
-            change_coins: List[Coin] = addition_dict[wid] if wid in addition_dict else []
+            potential_change_coins: List[Coin] = addition_dict[wid] if wid in addition_dict else []
+            change_coins: List[Coin] = [c for c in potential_change_coins if c.parent_coin_info in all_removals]
             change_amount: int = sum(c.amount for c in change_coins)
             sent_amount: int = removed - change_amount
             txs.append(
@@ -467,15 +585,17 @@ class TradeManager:
     async def respond_to_offer(self, offer: Offer, fee=uint64(0)) -> Tuple[bool, Optional[TradeRecord], Optional[str]]:
         take_offer_dict: Dict[Union[bytes32, int], int] = {}
         arbitrage: Dict[Optional[bytes32], int] = offer.arbitrage()
+
         for asset_id, amount in arbitrage.items():
             if asset_id is None:
                 wallet = self.wallet_state_manager.main_wallet
                 key: Union[bytes32, int] = int(wallet.id())
             else:
+                # ATTENTION: new wallets
                 wallet = await self.wallet_state_manager.get_wallet_for_asset_id(asset_id.hex())
                 if wallet is None and amount < 0:
-                    return False, None, f"Do not have a CAT of asset ID: {asset_id} to fulfill offer"
-                elif wallet is None:
+                    return False, None, f"Do not have a wallet for asset ID: {asset_id} to fulfill offer"
+                elif wallet is None or wallet.type() == WalletType.NFT:
                     key = asset_id
                 else:
                     key = int(wallet.id())
@@ -485,15 +605,14 @@ class TradeManager:
         valid: bool = await self.check_offer_validity(offer)
         if not valid:
             return False, None, "This offer is no longer valid"
-
-        success, take_offer, error = await self._create_offer_for_ids(take_offer_dict, fee=fee)
+        success, take_offer, error = await self._create_offer_for_ids(take_offer_dict, offer.driver_dict, fee=fee)
         if not success or take_offer is None:
             return False, None, error
 
         complete_offer = Offer.aggregate([offer, take_offer])
         assert complete_offer.is_valid()
-        final_spend_bundle: SpendBundle = complete_offer.to_valid_spend()
 
+        final_spend_bundle: SpendBundle = complete_offer.to_valid_spend()
         await self.maybe_create_wallets_for_offer(complete_offer)
 
         tx_records: List[TransactionRecord] = await self.calculate_tx_records_for_offer(complete_offer, True)
@@ -538,3 +657,46 @@ class TradeManager:
             await self.wallet_state_manager.add_transaction(tx)
 
         return True, trade_record, None
+
+    async def check_for_special_offer_making(
+        self,
+        offer_dict: Dict[Optional[bytes32], int],
+        driver_dict: Dict[bytes32, PuzzleInfo],
+        fee: uint64 = uint64(0),
+    ) -> Optional[Offer]:
+
+        for puzzle_info in driver_dict.values():
+            if (
+                puzzle_info.check_type(
+                    [
+                        AssetType.SINGLETON.value,
+                        AssetType.METADATA.value,
+                        AssetType.OWNERSHIP.value,
+                    ]
+                )
+                and isinstance(puzzle_info.also().also()["transfer_program"], PuzzleInfo)  # type: ignore
+                and puzzle_info.also().also()["transfer_program"].type()  # type: ignore
+                == AssetType.ROYALTY_TRANSFER_PROGRAM.value
+            ):
+                return await NFTWallet.make_nft1_offer(self.wallet_state_manager, offer_dict, driver_dict, fee)
+        return None
+
+    def check_for_owner_change_in_drivers(self, puzzle_info: PuzzleInfo, driver_info: PuzzleInfo) -> bool:
+        if puzzle_info.check_type(
+            [
+                AssetType.SINGLETON.value,
+                AssetType.METADATA.value,
+                AssetType.OWNERSHIP.value,
+            ]
+        ) and driver_info.check_type(
+            [
+                AssetType.SINGLETON.value,
+                AssetType.METADATA.value,
+                AssetType.OWNERSHIP.value,
+            ]
+        ):
+            old_owner = driver_info.also().also().info["owner"]  # type: ignore
+            puzzle_info.also().also().info["owner"] = old_owner  # type: ignore
+            if driver_info == puzzle_info:
+                return True
+        return False
