@@ -1,30 +1,33 @@
 from dataclasses import dataclass
-from typing import List, Optional, Dict, Set, Tuple
+from typing import Any, Dict, List, Optional, Set, Tuple
+
 from blspy import G2Element
+from clvm_tools.binutils import disassemble
 
 from chia.types.blockchain_format.sized_bytes import bytes32
-from chia.types.blockchain_format.coin import Coin
+from chia.types.blockchain_format.coin import Coin, coin_as_list
 from chia.types.blockchain_format.program import Program
 from chia.types.announcement import Announcement
 from chia.types.coin_spend import CoinSpend
 from chia.types.spend_bundle import SpendBundle
-from chia.util.bech32m import bech32_encode, bech32_decode, convertbits
+from chia.util.bech32m import bech32_decode, bech32_encode, convertbits
 from chia.util.ints import uint64
+from chia.wallet.outer_puzzles import (
+    construct_puzzle,
+    create_asset_id,
+    match_puzzle,
+    solve_puzzle,
+    get_inner_puzzle,
+    get_inner_solution,
+)
+from chia.wallet.payment import Payment
+from chia.wallet.puzzle_drivers import PuzzleInfo, Solver
+from chia.wallet.puzzles.load_clvm import load_clvm
 from chia.wallet.util.puzzle_compression import (
     compress_object_with_puzzles,
     decompress_object_with_puzzles,
     lowest_best_version,
 )
-from chia.wallet.cat_wallet.cat_utils import (
-    CAT_MOD,
-    SpendableCAT,
-    construct_cat_puzzle,
-    match_cat_puzzle,
-    unsigned_spend_bundle_for_spendable_cats,
-)
-from chia.wallet.lineage_proof import LineageProof
-from chia.wallet.puzzles.load_clvm import load_clvm
-from chia.wallet.payment import Payment
 
 OFFER_MOD = load_clvm("settlement_payments.clvm")
 ZERO_32 = bytes32([0] * 32)
@@ -48,6 +51,7 @@ class Offer:
         Optional[bytes32], List[NotarizedPayment]
     ]  # The key is the asset id of the asset being requested
     bundle: SpendBundle
+    driver_dict: Dict[bytes32, PuzzleInfo]  # asset_id -> asset driver
 
     @staticmethod
     def ph():
@@ -60,27 +64,29 @@ class Offer:
     ) -> Dict[Optional[bytes32], List[NotarizedPayment]]:
         # This sort should be reproducible in CLVM with `>s`
         sorted_coins: List[Coin] = sorted(coins, key=Coin.name)
-        sorted_coin_list: List[List] = [c.as_list() for c in sorted_coins]
+        sorted_coin_list: List[List] = [coin_as_list(c) for c in sorted_coins]
         nonce: bytes32 = Program.to(sorted_coin_list).get_tree_hash()
 
         notarized_payments: Dict[Optional[bytes32], List[NotarizedPayment]] = {}
-        for tail_hash, payments in requested_payments.items():
-            notarized_payments[tail_hash] = []
+        for asset_id, payments in requested_payments.items():
+            notarized_payments[asset_id] = []
             for p in payments:
                 puzzle_hash, amount, memos = tuple(p.as_condition_args())
-                notarized_payments[tail_hash].append(NotarizedPayment(puzzle_hash, amount, memos, nonce))
+                notarized_payments[asset_id].append(NotarizedPayment(puzzle_hash, amount, memos, nonce))
 
         return notarized_payments
 
     # The announcements returned from this function must be asserted in whatever spend bundle is created by the wallet
     @staticmethod
     def calculate_announcements(
-        notarized_payments: Dict[Optional[bytes32], List[NotarizedPayment]],
+        notarized_payments: Dict[Optional[bytes32], List[NotarizedPayment]], driver_dict: Dict[bytes32, PuzzleInfo]
     ) -> List[Announcement]:
         announcements: List[Announcement] = []
-        for tail, payments in notarized_payments.items():
-            if tail is not None:
-                settlement_ph: bytes32 = construct_cat_puzzle(CAT_MOD, tail, OFFER_MOD).get_tree_hash()
+        for asset_id, payments in notarized_payments.items():
+            if asset_id is not None:
+                if asset_id not in driver_dict:
+                    raise ValueError("Cannot calculate announcements without driver of requested item")
+                settlement_ph: bytes32 = construct_puzzle(driver_dict[asset_id], OFFER_MOD).get_tree_hash()
             else:
                 settlement_ph = OFFER_MOD.get_tree_hash()
 
@@ -103,33 +109,75 @@ class Offer:
             if len(set(payment_programs)) != len(payment_programs):
                 raise ValueError("Bundle has duplicate requested payments")
 
+        # Verify we have a type for every kind of asset
+        for asset_id in self.requested_payments:
+            if asset_id is not None and asset_id not in self.driver_dict:
+                raise ValueError("Offer does not have enough driver information about the requested payments")
+
+    def additions(self) -> List[Coin]:
+        final_list: List[Coin] = []
+        for cs in self.bundle.coin_spends:
+            try:
+                final_list.extend(cs.additions())
+            except Exception:
+                pass
+        return final_list
+
+    def removals(self) -> List[Coin]:
+        return self.bundle.removals()
+
+    def incomplete_spends(self) -> List[CoinSpend]:
+        final_list: List[CoinSpend] = []
+        for cs in self.bundle.coin_spends:
+            try:
+                cs.additions()
+            except Exception:
+                final_list.append(cs)
+        return final_list
+
     # This method does not get every coin that is being offered, only the `settlement_payment` children
+    # It's also a little heuristic, but it should get most things
     def get_offered_coins(self) -> Dict[Optional[bytes32], List[Coin]]:
         offered_coins: Dict[Optional[bytes32], List[Coin]] = {}
 
-        for addition in self.bundle.additions():
-            # Get the parent puzzle
-            parent_puzzle: Program = list(
-                filter(lambda cs: cs.coin.name() == addition.parent_coin_info, self.bundle.coin_spends)
-            )[0].puzzle_reveal.to_program()
+        OFFER_HASH: bytes32 = OFFER_MOD.get_tree_hash()
+        for parent_spend in self.bundle.coin_spends:
+            coins_for_this_spend: List[Coin] = []
 
-            # Determine it's TAIL (or lack of)
-            matched, curried_args = match_cat_puzzle(parent_puzzle)
-            tail_hash: Optional[bytes32] = None
-            if matched:
-                _, tail_hash_program, _ = curried_args
-                tail_hash = bytes32(tail_hash_program.as_python())
-                offer_ph: bytes32 = construct_cat_puzzle(CAT_MOD, tail_hash, OFFER_MOD).get_tree_hash()
+            parent_puzzle: Program = parent_spend.puzzle_reveal.to_program()
+            parent_solution: Program = parent_spend.solution.to_program()
+            additions: List[Coin] = [a for a in parent_spend.additions() if a not in self.bundle.removals()]
+
+            puzzle_driver = match_puzzle(parent_puzzle)
+            if puzzle_driver is not None:
+                asset_id = create_asset_id(puzzle_driver)
+                inner_puzzle: Optional[Program] = get_inner_puzzle(puzzle_driver, parent_puzzle)
+                inner_solution: Optional[Program] = get_inner_solution(puzzle_driver, parent_solution)
+                assert inner_puzzle is not None and inner_solution is not None
+                conditions: Program = inner_puzzle.run(inner_solution)
+                for condition in conditions.as_iter():
+                    if condition.first() == 51 and condition.rest().first() == OFFER_HASH:
+                        additions_w_amount: List[Coin] = [
+                            a for a in additions if a.amount == condition.rest().rest().first().as_int()
+                        ]
+                        if len(additions_w_amount) == 1:
+                            coins_for_this_spend.append(additions_w_amount[0])
+                        else:
+                            additions_w_amount_and_puzhash: List[Coin] = [
+                                a
+                                for a in additions_w_amount
+                                if a.puzzle_hash
+                                == construct_puzzle(puzzle_driver, OFFER_HASH).get_tree_hash(OFFER_HASH)  # type: ignore
+                            ]
+                            if len(additions_w_amount_and_puzhash) == 1:
+                                coins_for_this_spend.append(additions_w_amount_and_puzhash[0])
             else:
-                tail_hash = None
-                offer_ph = OFFER_MOD.get_tree_hash()
+                asset_id = None
+                coins_for_this_spend.extend([a for a in additions if a.puzzle_hash == OFFER_HASH])
 
-            # Check if the puzzle_hash matches the hypothetical `settlement_payments` puzzle hash
-            if addition.puzzle_hash == offer_ph:
-                if tail_hash in offered_coins:
-                    offered_coins[tail_hash].append(addition)
-                else:
-                    offered_coins[tail_hash] = [addition]
+            if coins_for_this_spend != []:
+                offered_coins.setdefault(asset_id, [])
+                offered_coins[asset_id].extend(coins_for_this_spend)
 
         return offered_coins
 
@@ -160,12 +208,12 @@ class Offer:
         return arbitrage_dict
 
     # This is a method mostly for the UI that creates a JSON summary of the offer
-    def summary(self) -> Tuple[Dict[str, int], Dict[str, int]]:
+    def summary(self) -> Tuple[Dict[str, int], Dict[str, int], Dict[str, Dict[str, Any]]]:
         offered_amounts: Dict[Optional[bytes32], int] = self.get_offered_amounts()
         requested_amounts: Dict[Optional[bytes32], int] = self.get_requested_amounts()
 
-        def keys_to_strings(dic: Dict[Optional[bytes32], int]) -> Dict[str, int]:
-            new_dic: Dict[str, int] = {}
+        def keys_to_strings(dic: Dict[Optional[bytes32], Any]) -> Dict[str, Any]:
+            new_dic: Dict[str, Any] = {}
             for key in dic:
                 if key is None:
                     new_dic["xch"] = dic[key]
@@ -173,13 +221,17 @@ class Offer:
                     new_dic[key.hex()] = dic[key]
             return new_dic
 
-        return keys_to_strings(offered_amounts), keys_to_strings(requested_amounts)
+        driver_dict: Dict[str, Any] = {}
+        for key, value in self.driver_dict.items():
+            driver_dict[key.hex()] = value.info
+
+        return keys_to_strings(offered_amounts), keys_to_strings(requested_amounts), driver_dict
 
     # Also mostly for the UI, returns a dictionary of assets and how much of them is pended for this offer
     # This method is also imperfect for sufficiently complex spends
     def get_pending_amounts(self) -> Dict[str, int]:
-        all_additions: List[Coin] = self.bundle.additions()
-        all_removals: List[Coin] = self.bundle.removals()
+        all_additions: List[Coin] = self.additions()
+        all_removals: List[Coin] = self.removals()
         non_ephemeral_removals: List[Coin] = list(filter(lambda c: c not in all_additions, all_removals))
 
         pending_dict: Dict[str, int] = {}
@@ -203,13 +255,13 @@ class Offer:
 
     # This method returns all of the coins that are being used in the offer (without which it would be invalid)
     def get_involved_coins(self) -> List[Coin]:
-        additions = self.bundle.additions()
-        return list(filter(lambda c: c not in additions, self.bundle.removals()))
+        additions = self.additions()
+        return list(filter(lambda c: c not in additions, self.removals()))
 
     # This returns the non-ephemeral removal that is an ancestor of the specified coin
     # This should maybe move to the SpendBundle object at some point
     def get_root_removal(self, coin: Coin) -> Coin:
-        all_removals: Set[Coin] = set(self.bundle.removals())
+        all_removals: Set[Coin] = set(self.removals())
         all_removal_ids: Set[bytes32] = {c.name() for c in all_removals}
         non_ephemeral_removals: Set[Coin] = {
             c for c in all_removals if c.parent_coin_info not in {r.name() for r in all_removals}
@@ -234,6 +286,7 @@ class Offer:
     def aggregate(cls, offers: List["Offer"]) -> "Offer":
         total_requested_payments: Dict[Optional[bytes32], List[NotarizedPayment]] = {}
         total_bundle = SpendBundle([], G2Element())
+        total_driver_dict: Dict[bytes32, PuzzleInfo] = {}
         for offer in offers:
             # First check for any overlap in inputs
             total_inputs: Set[Coin] = {cs.coin for cs in total_bundle.coin_spends}
@@ -242,15 +295,20 @@ class Offer:
                 raise ValueError("The aggregated offers overlap inputs")
 
             # Next, do the aggregation
-            for tail, payments in offer.requested_payments.items():
-                if tail in total_requested_payments:
-                    total_requested_payments[tail].extend(payments)
+            for asset_id, payments in offer.requested_payments.items():
+                if asset_id in total_requested_payments:
+                    total_requested_payments[asset_id].extend(payments)
                 else:
-                    total_requested_payments[tail] = payments
+                    total_requested_payments[asset_id] = payments
+
+            for key, value in offer.driver_dict.items():
+                if key in total_driver_dict and total_driver_dict[key] != value:
+                    raise ValueError(f"The offers to aggregate disagree on the drivers for {key.hex()}")
 
             total_bundle = SpendBundle.aggregate([total_bundle, offer.bundle])
+            total_driver_dict.update(offer.driver_dict)
 
-        return cls(total_requested_payments, total_bundle)
+        return cls(total_requested_payments, total_bundle, total_driver_dict)
 
     # Validity is defined by having enough funds within the offer to satisfy both sides
     def is_valid(self) -> bool:
@@ -263,55 +321,82 @@ class Offer:
             raise ValueError("Offer is currently incomplete")
 
         completion_spends: List[CoinSpend] = []
-        for tail_hash, payments in self.requested_payments.items():
-            offered_coins: List[Coin] = self.get_offered_coins()[tail_hash]
+        for asset_id, payments in self.requested_payments.items():
+            offered_coins: List[Coin] = self.get_offered_coins()[asset_id]
 
             # Because of CAT supply laws, we must specify a place for the leftovers to go
-            arbitrage_amount: int = self.arbitrage()[tail_hash]
+            arbitrage_amount: int = self.arbitrage()[asset_id]
             all_payments: List[NotarizedPayment] = payments.copy()
             if arbitrage_amount > 0:
                 assert arbitrage_amount is not None
                 assert arbitrage_ph is not None
                 all_payments.append(NotarizedPayment(arbitrage_ph, uint64(arbitrage_amount), []))
 
+            # Some assets need to know about siblings so we need to collect all spends first to be able to use them
+            coin_to_spend_dict: Dict[Coin, CoinSpend] = {}
+            coin_to_solution_dict: Dict[Coin, Program] = {}
             for coin in offered_coins:
+                parent_spend: CoinSpend = list(
+                    filter(lambda cs: cs.coin.name() == coin.parent_coin_info, self.bundle.coin_spends)
+                )[0]
+                coin_to_spend_dict[coin] = parent_spend
+
                 inner_solutions = []
                 if coin == offered_coins[0]:
                     nonces: List[bytes32] = [p.nonce for p in all_payments]
                     for nonce in list(dict.fromkeys(nonces)):  # dedup without messing with order
                         nonce_payments: List[NotarizedPayment] = list(filter(lambda p: p.nonce == nonce, all_payments))
                         inner_solutions.append((nonce, [np.as_condition_args() for np in nonce_payments]))
+                coin_to_solution_dict[coin] = Program.to(inner_solutions)
 
-                if tail_hash:
-                    # CATs have a special way to be solved so we have to do some calculation before getting the solution
-                    parent_spend: CoinSpend = list(
-                        filter(lambda cs: cs.coin.name() == coin.parent_coin_info, self.bundle.coin_spends)
-                    )[0]
-                    parent_coin: Coin = parent_spend.coin
-                    matched, curried_args = match_cat_puzzle(parent_spend.puzzle_reveal.to_program())
-                    assert matched
-                    _, _, inner_puzzle = curried_args
-                    spendable_cat = SpendableCAT(
-                        coin,
-                        tail_hash,
-                        OFFER_MOD,
-                        Program.to(inner_solutions),
-                        lineage_proof=LineageProof(
-                            parent_coin.parent_coin_info, inner_puzzle.get_tree_hash(), parent_coin.amount
+            for coin in offered_coins:
+                if asset_id:
+                    siblings: str = "("
+                    sibling_spends: str = "("
+                    sibling_puzzles: str = "("
+                    sibling_solutions: str = "("
+                    disassembled_offer_mod: str = disassemble(OFFER_MOD)
+                    for sibling_coin in offered_coins:
+                        if sibling_coin != coin:
+                            siblings += (
+                                "0x"
+                                + sibling_coin.parent_coin_info.hex()
+                                + sibling_coin.puzzle_hash.hex()
+                                + bytes(sibling_coin.amount).hex()
+                            )
+                            sibling_spends += "0x" + bytes(coin_to_spend_dict[sibling_coin]).hex() + ")"
+                            sibling_puzzles += disassembled_offer_mod
+                            sibling_solutions += disassemble(coin_to_solution_dict[sibling_coin])
+                    siblings += ")"
+                    sibling_spends += ")"
+                    sibling_puzzles += ")"
+                    sibling_solutions += ")"
+
+                    solution: Program = solve_puzzle(
+                        self.driver_dict[asset_id],
+                        Solver(
+                            {
+                                "coin": "0x"
+                                + coin.parent_coin_info.hex()
+                                + coin.puzzle_hash.hex()
+                                + bytes(coin.amount).hex(),
+                                "parent_spend": "0x" + bytes(coin_to_spend_dict[coin]).hex(),
+                                "siblings": siblings,
+                                "sibling_spends": sibling_spends,
+                                "sibling_puzzles": sibling_puzzles,
+                                "sibling_solutions": sibling_solutions,
+                            }
                         ),
-                    )
-                    solution: Program = (
-                        unsigned_spend_bundle_for_spendable_cats(CAT_MOD, [spendable_cat])
-                        .coin_spends[0]
-                        .solution.to_program()
+                        OFFER_MOD,
+                        Program.to(coin_to_solution_dict[coin]),
                     )
                 else:
-                    solution = Program.to(inner_solutions)
+                    solution = Program.to(coin_to_solution_dict[coin])
 
                 completion_spends.append(
                     CoinSpend(
                         coin,
-                        construct_cat_puzzle(CAT_MOD, tail_hash, OFFER_MOD) if tail_hash else OFFER_MOD,
+                        construct_puzzle(self.driver_dict[asset_id], OFFER_MOD) if asset_id else OFFER_MOD,
                         solution,
                     )
                 )
@@ -321,8 +406,8 @@ class Offer:
     def to_spend_bundle(self) -> SpendBundle:
         # Before we serialze this as a SpendBundle, we need to serialze the `requested_payments` as dummy CoinSpends
         additional_coin_spends: List[CoinSpend] = []
-        for tail_hash, payments in self.requested_payments.items():
-            puzzle_reveal: Program = construct_cat_puzzle(CAT_MOD, tail_hash, OFFER_MOD) if tail_hash else OFFER_MOD
+        for asset_id, payments in self.requested_payments.items():
+            puzzle_reveal: Program = construct_puzzle(self.driver_dict[asset_id], OFFER_MOD) if asset_id else OFFER_MOD
             inner_solutions = []
             nonces: List[bytes32] = [p.nonce for p in payments]
             for nonce in list(dict.fromkeys(nonces)):  # dedup without messing with order
@@ -352,16 +437,17 @@ class Offer:
     def from_spend_bundle(cls, bundle: SpendBundle) -> "Offer":
         # Because of the `to_spend_bundle` method, we need to parse the dummy CoinSpends as `requested_payments`
         requested_payments: Dict[Optional[bytes32], List[NotarizedPayment]] = {}
+        driver_dict: Dict[bytes32, PuzzleInfo] = {}
         leftover_coin_spends: List[CoinSpend] = []
         for coin_spend in bundle.coin_spends:
+            driver = match_puzzle(coin_spend.puzzle_reveal.to_program())
+            if driver is not None:
+                asset_id = create_asset_id(driver)
+                assert asset_id is not None
+                driver_dict[asset_id] = driver
+            else:
+                asset_id = None
             if coin_spend.coin.parent_coin_info == ZERO_32:
-                matched, curried_args = match_cat_puzzle(coin_spend.puzzle_reveal.to_program())
-                if matched:
-                    _, tail_hash_program, _ = curried_args
-                    tail_hash: Optional[bytes32] = bytes32(tail_hash_program.as_python())
-                else:
-                    tail_hash = None
-
                 notarized_payments: List[NotarizedPayment] = []
                 for payment_group in coin_spend.solution.to_program().as_iter():
                     nonce = bytes32(payment_group.first().as_python())
@@ -369,12 +455,12 @@ class Offer:
                     notarized_payments.extend(
                         [NotarizedPayment.from_condition_and_nonce(condition, nonce) for condition in payment_args_list]
                     )
-                requested_payments[tail_hash] = notarized_payments
 
+                requested_payments[asset_id] = notarized_payments
             else:
                 leftover_coin_spends.append(coin_spend)
 
-        return cls(requested_payments, SpendBundle(leftover_coin_spends, bundle.aggregated_signature))
+        return cls(requested_payments, SpendBundle(leftover_coin_spends, bundle.aggregated_signature), driver_dict)
 
     def name(self) -> bytes32:
         return self.to_spend_bundle().name()
