@@ -4,6 +4,7 @@ import aiosqlite
 
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.util.db_wrapper import DBWrapper
+from chia.util.ints import uint32
 
 
 class WalletInterestedStore:
@@ -26,14 +27,21 @@ class WalletInterestedStore:
         await self.db_connection.execute(
             "CREATE TABLE IF NOT EXISTS interested_puzzle_hashes(puzzle_hash text PRIMARY KEY, wallet_id integer)"
         )
+
+        # Table for unknown CATs
+        fields = "asset_id text PRIMARY KEY, name text, first_seen_height integer, sender_puzzle_hash text"
+        await self.db_connection.execute(f"CREATE TABLE IF NOT EXISTS unacknowledged_asset_tokens({fields})")
+
         await self.db_connection.commit()
         return self
 
     async def _clear_database(self):
-        cursor = await self.db_connection.execute("DELETE FROM puzzle_hashes")
+        cursor = await self.db_connection.execute("DELETE FROM interested_puzzle_hashes")
         await cursor.close()
         cursor = await self.db_connection.execute("DELETE FROM interested_coins")
         await cursor.close()
+        cursor = await self.db_connection.execute("DELETE FROM unacknowledged_asset_tokens")
+        await cursor.close()
         await self.db_connection.commit()
 
     async def get_interested_coin_ids(self) -> List[bytes32]:
@@ -97,3 +105,52 @@ class WalletInterestedStore:
             if not in_transaction:
                 await self.db_connection.commit()
                 self.db_wrapper.lock.release()
+
+    async def add_unacknowledged_token(
+        self,
+        asset_id: bytes32,
+        name: str,
+        first_seen_height: Optional[uint32],
+        sender_puzzle_hash: bytes32,
+        in_transaction: bool = True,
+    ) -> None:
+        """
+        Add an unacknowledged CAT to the database. It will only be inserted once at the first time.
+        :param asset_id: CAT asset ID
+        :param name: Name of the CAT, for now it will be unknown until we integrate the CAT name service
+        :param first_seen_height: The block height of the wallet received this CAT in the first time
+        :param sender_puzzle_hash: The puzzle hash of the sender
+        :param in_transaction: In transaction or not
+        :return: None
+        """
+        if not in_transaction:
+            await self.db_wrapper.lock.acquire()
+        try:
+            cursor = await self.db_connection.execute(
+                "INSERT OR IGNORE INTO unacknowledged_asset_tokens VALUES (?, ?, ?, ?)",
+                (
+                    asset_id.hex(),
+                    name,
+                    first_seen_height if first_seen_height is not None else 0,
+                    sender_puzzle_hash.hex(),
+                ),
+            )
+            await cursor.close()
+        finally:
+            if not in_transaction:
+                await self.db_connection.commit()
+                self.db_wrapper.lock.release()
+
+    async def get_unacknowledged_tokens(self) -> List:
+        """
+        Get a list of all unacknowledged CATs
+        :return: A json style list of unacknowledged CATs
+        """
+        cursor = await self.db_connection.execute(
+            "SELECT asset_id, name, first_seen_height, sender_puzzle_hash FROM unacknowledged_asset_tokens"
+        )
+        cats = await cursor.fetchall()
+        return [
+            {"asset_id": cat[0], "name": cat[1], "first_seen_height": cat[2], "sender_puzzle_hash": cat[3]}
+            for cat in cats
+        ]
