@@ -71,8 +71,8 @@ def generate_replacements(conf, dir):
         "CHECK_RESOURCE_USAGE": read_file(
             Path(root_path / "runner_templates/check-resource-usage.include.yml")
         ).rstrip(),
-        "DISABLE_PYTEST_MONITOR": "",
-        "TEST_DIR": "",
+        "ENABLE_PYTEST_MONITOR": "",
+        "TEST_FILES": "",
         "TEST_NAME": "",
         "PYTEST_PARALLEL_ARGS": "",
     }
@@ -88,13 +88,16 @@ def generate_replacements(conf, dir):
         replacements["INSTALL_TIMELORD"] = "# Omitted installing Timelord"
     if conf["job_timeout"]:
         replacements["JOB_TIMEOUT"] = str(conf["job_timeout"])
-    replacements["TEST_DIR"] = "/".join([*dir.relative_to(root_path.parent).parts, "test_*.py"])
+    test_files = sorted(dir.glob("test_*.py"))
+    test_file_paths = [file.relative_to(root_path.parent).as_posix() for file in test_files]
+    replacements["TEST_FILES"] = " ".join(test_file_paths)
     replacements["TEST_NAME"] = test_name(dir)
     if "test_name" in conf:
         replacements["TEST_NAME"] = conf["test_name"]
-    if not conf["check_resource_usage"]:
+    if conf["check_resource_usage"]:
+        replacements["ENABLE_PYTEST_MONITOR"] = "-p monitor"
+    else:
         replacements["CHECK_RESOURCE_USAGE"] = "# Omitted resource usage check"
-        replacements["DISABLE_PYTEST_MONITOR"] = "-p no:monitor"
     for var in conf["custom_vars"]:
         replacements[var] = conf[var] if var in conf else ""
     return replacements
