@@ -118,26 +118,29 @@ class TestBlockStore:
                 # make sure all block heights are unique
                 assert len(set(ret)) == count
 
-            for block in blocks:
-                async with db_wrapper.db.execute(
-                    "SELECT in_main_chain FROM full_blocks WHERE header_hash=?", (block.header_hash,)
-                ) as cursor:
-                    rows = await cursor.fetchall()
-                    assert len(rows) == 1
-                    assert rows[0][0]
+            async with db_wrapper.read_db() as conn:
+                for block in blocks:
+                    async with conn.execute(
+                        "SELECT in_main_chain FROM full_blocks WHERE header_hash=?", (block.header_hash,)
+                    ) as cursor:
+                        rows = await cursor.fetchall()
+                        assert len(rows) == 1
+                        assert rows[0][0]
 
             await block_store.rollback(5)
 
             count = 0
-            for block in blocks:
-                async with db_wrapper.db.execute(
-                    "SELECT in_main_chain FROM full_blocks WHERE header_hash=? ORDER BY height", (block.header_hash,)
-                ) as cursor:
-                    rows = await cursor.fetchall()
-                    print(count, rows)
-                    assert len(rows) == 1
-                    assert rows[0][0] == (count <= 5)
-                count += 1
+            async with db_wrapper.read_db() as conn:
+                for block in blocks:
+                    async with conn.execute(
+                        "SELECT in_main_chain FROM full_blocks WHERE header_hash=? ORDER BY height",
+                        (block.header_hash,),
+                    ) as cursor:
+                        rows = await cursor.fetchall()
+                        print(count, rows)
+                        assert len(rows) == 1
+                        assert rows[0][0] == (count <= 5)
+                    count += 1
 
     @pytest.mark.asyncio
     async def test_count_compactified_blocks(self, bt, tmp_dir, db_version):
