@@ -3,7 +3,6 @@ These are quick-to-run test that check spends can be added to the blockchain whe
 or that they're failing for the right reason when they're invalid.
 """
 
-import atexit
 import logging
 import time
 
@@ -15,7 +14,6 @@ from blspy import G2Element
 
 from clvm_tools.binutils import assemble
 
-from chia.consensus.constants import ConsensusConstants
 from chia.types.announcement import Announcement
 from chia.types.blockchain_format.program import Program
 from chia.types.coin_record import CoinRecord
@@ -25,7 +23,7 @@ from chia.types.full_block import FullBlock
 from chia.types.spend_bundle import SpendBundle
 from chia.util.errors import Err
 from chia.util.ints import uint32
-from tests.block_tools import create_block_tools, test_constants
+from tests.block_tools import BlockTools
 from tests.util.keyring import TempKeyring
 
 from .ram_db import create_ram_blockchain
@@ -36,12 +34,6 @@ def cleanup_keyring(keyring: TempKeyring):
     keyring.cleanup()
 
 
-temp_keyring = TempKeyring()
-keychain = temp_keyring.get_keychain()
-atexit.register(cleanup_keyring, temp_keyring)  # Attempt to cleanup the temp keychain
-bt = create_block_tools(constants=test_constants, keychain=keychain)
-
-
 log = logging.getLogger(__name__)
 
 
@@ -52,7 +44,7 @@ EASY_PUZZLE = Program.to(assemble("1"))
 EASY_PUZZLE_HASH = EASY_PUZZLE.get_tree_hash()
 
 
-def initial_blocks(block_count: int = 4) -> List[FullBlock]:
+async def initial_blocks(bt, block_count: int = 4) -> List[FullBlock]:
     blocks = bt.get_consecutive_blocks(
         block_count,
         guarantee_transaction_block=True,
@@ -63,7 +55,7 @@ def initial_blocks(block_count: int = 4) -> List[FullBlock]:
 
 
 async def check_spend_bundle_validity(
-    constants: ConsensusConstants,
+    bt: BlockTools,
     blocks: List[FullBlock],
     spend_bundle: SpendBundle,
     expected_err: Optional[Err] = None,
@@ -73,6 +65,7 @@ async def check_spend_bundle_validity(
     `SpendBundle`, and then invokes `receive_block` to ensure that it's accepted (if `expected_err=None`)
     or fails with the correct error code.
     """
+    constants = bt.constants
     db_wrapper, blockchain = await create_ram_blockchain(constants)
     try:
         for block in blocks:
@@ -106,9 +99,9 @@ async def check_spend_bundle_validity(
 
 
 async def check_conditions(
-    condition_solution: Program, expected_err: Optional[Err] = None, spend_reward_index: int = -2
+    bt: BlockTools, condition_solution: Program, expected_err: Optional[Err] = None, spend_reward_index: int = -2
 ):
-    blocks = initial_blocks()
+    blocks = await initial_blocks(bt)
     coin = list(blocks[spend_reward_index].get_included_reward_coins())[0]
 
     coin_spend = CoinSpend(coin, EASY_PUZZLE, condition_solution)
@@ -116,63 +109,63 @@ async def check_conditions(
 
     # now let's try to create a block with the spend bundle and ensure that it doesn't validate
 
-    await check_spend_bundle_validity(bt.constants, blocks, spend_bundle, expected_err=expected_err)
+    await check_spend_bundle_validity(bt, blocks, spend_bundle, expected_err=expected_err)
 
 
 class TestConditions:
     @pytest.mark.asyncio
-    async def test_invalid_block_age(self):
+    async def test_invalid_block_age(self, bt):
         conditions = Program.to(assemble(f"(({ConditionOpcode.ASSERT_HEIGHT_RELATIVE[0]} 2))"))
-        await check_conditions(conditions, expected_err=Err.ASSERT_HEIGHT_RELATIVE_FAILED)
+        await check_conditions(bt, conditions, expected_err=Err.ASSERT_HEIGHT_RELATIVE_FAILED)
 
     @pytest.mark.asyncio
-    async def test_valid_block_age(self):
+    async def test_valid_block_age(self, bt):
         conditions = Program.to(assemble(f"(({ConditionOpcode.ASSERT_HEIGHT_RELATIVE[0]} 1))"))
-        await check_conditions(conditions)
+        await check_conditions(bt, conditions)
 
     @pytest.mark.asyncio
-    async def test_invalid_block_height(self):
+    async def test_invalid_block_height(self, bt):
         conditions = Program.to(assemble(f"(({ConditionOpcode.ASSERT_HEIGHT_ABSOLUTE[0]} 4))"))
-        await check_conditions(conditions, expected_err=Err.ASSERT_HEIGHT_ABSOLUTE_FAILED)
+        await check_conditions(bt, conditions, expected_err=Err.ASSERT_HEIGHT_ABSOLUTE_FAILED)
 
     @pytest.mark.asyncio
-    async def test_valid_block_height(self):
+    async def test_valid_block_height(self, bt):
         conditions = Program.to(assemble(f"(({ConditionOpcode.ASSERT_HEIGHT_ABSOLUTE[0]} 3))"))
-        await check_conditions(conditions)
+        await check_conditions(bt, conditions)
 
     @pytest.mark.asyncio
-    async def test_invalid_my_id(self):
-        blocks = initial_blocks()
+    async def test_invalid_my_id(self, bt):
+        blocks = await initial_blocks(bt)
         coin = list(blocks[-2].get_included_reward_coins())[0]
         wrong_name = bytearray(coin.name())
         wrong_name[-1] ^= 1
         conditions = Program.to(assemble(f"(({ConditionOpcode.ASSERT_MY_COIN_ID[0]} 0x{wrong_name.hex()}))"))
-        await check_conditions(conditions, expected_err=Err.ASSERT_MY_COIN_ID_FAILED)
+        await check_conditions(bt, conditions, expected_err=Err.ASSERT_MY_COIN_ID_FAILED)
 
     @pytest.mark.asyncio
-    async def test_valid_my_id(self):
-        blocks = initial_blocks()
+    async def test_valid_my_id(self, bt):
+        blocks = await initial_blocks(bt)
         coin = list(blocks[-2].get_included_reward_coins())[0]
         conditions = Program.to(assemble(f"(({ConditionOpcode.ASSERT_MY_COIN_ID[0]} 0x{coin.name().hex()}))"))
-        await check_conditions(conditions)
+        await check_conditions(bt, conditions)
 
     @pytest.mark.asyncio
-    async def test_invalid_seconds_absolute(self):
+    async def test_invalid_seconds_absolute(self, bt):
         # TODO: make the test suite not use `time.time` so we can more accurately
         # set `time_now` to make it minimal while still failing
         time_now = int(time.time()) + 3000
         conditions = Program.to(assemble(f"(({ConditionOpcode.ASSERT_SECONDS_ABSOLUTE[0]} {time_now}))"))
-        await check_conditions(conditions, expected_err=Err.ASSERT_SECONDS_ABSOLUTE_FAILED)
+        await check_conditions(bt, conditions, expected_err=Err.ASSERT_SECONDS_ABSOLUTE_FAILED)
 
     @pytest.mark.asyncio
-    async def test_valid_seconds_absolute(self):
+    async def test_valid_seconds_absolute(self, bt):
         time_now = int(time.time())
         conditions = Program.to(assemble(f"(({ConditionOpcode.ASSERT_SECONDS_ABSOLUTE[0]} {time_now}))"))
-        await check_conditions(conditions)
+        await check_conditions(bt, conditions)
 
     @pytest.mark.asyncio
-    async def test_invalid_coin_announcement(self):
-        blocks = initial_blocks()
+    async def test_invalid_coin_announcement(self, bt):
+        blocks = await initial_blocks(bt)
         coin = list(blocks[-2].get_included_reward_coins())[0]
         announce = Announcement(coin.name(), b"test_bad")
         conditions = Program.to(
@@ -181,11 +174,11 @@ class TestConditions:
                 f"({ConditionOpcode.ASSERT_COIN_ANNOUNCEMENT[0]} 0x{announce.name().hex()}))"
             )
         )
-        await check_conditions(conditions, expected_err=Err.ASSERT_ANNOUNCE_CONSUMED_FAILED)
+        await check_conditions(bt, conditions, expected_err=Err.ASSERT_ANNOUNCE_CONSUMED_FAILED)
 
     @pytest.mark.asyncio
-    async def test_valid_coin_announcement(self):
-        blocks = initial_blocks()
+    async def test_valid_coin_announcement(self, bt):
+        blocks = await initial_blocks(bt)
         coin = list(blocks[-2].get_included_reward_coins())[0]
         announce = Announcement(coin.name(), b"test")
         conditions = Program.to(
@@ -194,10 +187,10 @@ class TestConditions:
                 f"({ConditionOpcode.ASSERT_COIN_ANNOUNCEMENT[0]} 0x{announce.name().hex()}))"
             )
         )
-        await check_conditions(conditions)
+        await check_conditions(bt, conditions)
 
     @pytest.mark.asyncio
-    async def test_invalid_puzzle_announcement(self):
+    async def test_invalid_puzzle_announcement(self, bt):
         announce = Announcement(EASY_PUZZLE_HASH, b"test_bad")
         conditions = Program.to(
             assemble(
@@ -205,10 +198,10 @@ class TestConditions:
                 f"({ConditionOpcode.ASSERT_PUZZLE_ANNOUNCEMENT[0]} 0x{announce.name().hex()}))"
             )
         )
-        await check_conditions(conditions, expected_err=Err.ASSERT_ANNOUNCE_CONSUMED_FAILED)
+        await check_conditions(bt, conditions, expected_err=Err.ASSERT_ANNOUNCE_CONSUMED_FAILED)
 
     @pytest.mark.asyncio
-    async def test_valid_puzzle_announcement(self):
+    async def test_valid_puzzle_announcement(self, bt):
         announce = Announcement(EASY_PUZZLE_HASH, b"test")
         conditions = Program.to(
             assemble(
@@ -216,4 +209,4 @@ class TestConditions:
                 f"({ConditionOpcode.ASSERT_PUZZLE_ANNOUNCEMENT[0]} 0x{announce.name().hex()}))"
             )
         )
-        await check_conditions(conditions)
+        await check_conditions(bt, conditions)
