@@ -112,7 +112,7 @@ class TestROM:
             puzzle_hash=bytes32.fromhex("9dcf97a184f32623d11a73124ceb99a5709b083721e878a16d78f596718ba7b2"),
             height_relative=None,
             seconds_relative=0,
-            create_coin=[(bytes([0] * 31 + [1]), 500, b"")],
+            create_coin=[(bytes([0] * 31 + [1]), 500, None)],
             agg_sig_me=[],
         )
 
